@@ -33,6 +33,287 @@
 #'      on data transformed data using MLE of \eqn{\lambda_P} and \eqn{\lambda_N}.
 #'  }
 #'
+#' @return  An S3 object of class \code{\link{fsrfan.object}} will be returned which is basically a list
+#'  containing the following elements:
+#'  \enumerate{
+#'  \item \code{la}: vector containing the values of lambda for which fan plot is constructed
+#'  \item \code{bs}: matrix of size \code{p X length(la)} containing the units forming
+#'      the initial subset for each value of lambda
+#'  \item \code{Score}: a matrix containing the values of the score test for
+#'      each value of the transformation parameter:
+#'      \itemize{
+#'      \item 1st col = fwd search index;
+#'      \item 2nd col = value of the score test in each step of the fwd search for la[1]
+#'      \item ...
+#'      }
+#'  \item \code{Scorep}: matrix containing the values of the score test for positive
+#'      observations for each value of the transformation parameter.
+#'
+#'      Note: this output is present only if input option \code{family='YJpn'} or \code{family='YJall'}.
+#'
+#'  \item \code{Scoren}: matrix containing the values of the score test for negative observations
+#'      for each value of the transformation parameter.
+#'
+#'      Note: this output is present only if input option 'family' is 'YJpn' or 'YJall'.
+#'
+#'  \item \code{Scoreb}: matrix containing the values of the score test for the joint
+#'      presence of both constructed variables (associated with positive and negative
+#'      observations) for each value of the transformation parameter. In this case
+#'      the reference distribution is the \eqn{F} with 2 and \code{subset_size - p}
+#'      degrees of freedom.
+#'
+#'      Note: this output is present only if input option \code{family='YJall'}.
+#'
+#'  \item \code{Un}: a three-dimensional array containing \code{length(la)} matrices of
+#'      size \code{retnUn=(n-init) X retpUn=11}. Each matrix contains
+#'      the unit(s) included in the subset at each step in the search associated
+#'      with the corresponding element of \code{la}.
+#'
+#'      REMARK: at each step the new subset is compared with the old subset.
+#'      \code{Un} contains the unit(s) present in the new subset but not in the old one.
+#'  }
+#'
+#' @references
+#'  Atkinson, A.C. and Riani, M. (2000), \emph{Robust Diagnostic Regression Analysis} Springer Verlag, New York.
+#'
+#'  Atkinson, A.C. and Riani, M. (2002), Tests in the fan plot for robust, diagnostic transformations in regression,
+#'      \emph{Chemometrics and Intelligent Laboratory Systems}, \bold{60}, pp. 87--100.
+#'
+#'  Atkinson, A.C. Riani, M. and  Corbellini A. (2019), The analysis of transformations for profit-and-loss data,
+#'      \emph{Journal of the Royal Statistical Society, Series C, "Applied Statistics"}, \bold{69}, pp. 251--275.
+#'      \doi{10.1111/rssc.12389}
+#'
+#'  Atkinson, A.C. Riani, M. and Corbellini A. (2021), The Box-Cox Transformation: Review and Extensions,
+#'      \emph{Statistical Science}, \bold{36}(2), pp. 239--255. \doi{10.1214/20-STS778}.
+#'
+#' @examples
+#'
+#' \dontrun{
+#'    data(wool)
+#'    XX <- wool
+#'    y <- XX[, ncol(XX)]
+#'    X <- XX[, 1:(ncol(XX)-1), drop=FALSE]
+#'
+#'    out <- fsrfan(X, y)                   # call 'fsrfan' with all default parameters
+#'    out <- fsrfan(cycles~., data=wool)    # use the formula interface
+#'
+#'    set.seed(10)
+#'    out <- fsrfan(cycles~., data=wool, plot=TRUE) # call 'fsrfan' and produce the plot
+#'    plot(out)                               # use the plot method on the fsrfan object
+#'    plot(out, conflev=c(0.9, 0.95, 0.99))   # change the confidence leel in the plot method
+#'
+#'  ##======================
+#'  ##
+#'  ##  fsrfan() with all default options.
+#'
+#'  ##  Store values of the score test statistic for the five most common
+#'  ##  values of $\lambda$. Produce also a fan plot and display it on the screen.
+#'  ##  Common part to all examples: load 'wool' data set.
+#'  data(wool)
+#'  head(wool)
+#'  dim(wool)
+#'  ##  The function fsrfan() stores the score test statistic.
+#'  ##  In this case we use the five most common values of lambda are considered
+#'
+#'  out <- fsrfan(cycles~., data=wool)
+#'  plot(out)
+#'  ##  fanplot(out)                # Not yet implemented in fsdaR
+#'
+#'  ##  The fan plot shows the log transformation is diffused throughout the data
+#'  ##  and does not depend on the presence of particular observations.
+
+#'  ##======================
+#'  ##
+#'  ##  Example specifying 'lambda'.
+#'  ##      Produce a fan plot for each value of 'lambda' in the vector 'la'.
+#'  ##      Extract in matrix 'Un' the units which entered the search in each step
+#'
+#'      data(wool)
+#'      out <- fsrfan(cycles~., data=wool, la=c(-1, -0.5, 0, 0.5), plot=TRUE)
+#'      plot(out)
+#'
+#'      out$Un[,2,]
+#'
+#'  ##======================
+#'  ##  Example specifying the confidence level and the initial starting point for monitoring.
+#'  ##  Construct the fan plot specifying the confidence level and the initial starting point
+#'  ##  for monitoring.
+#'      data(wool)
+#'      out <- fsrfan(cycles~., data=wool, init=ncol(wool)+1, nsamp=0, conflev=0.95, plots=TRUE)
+#'      plot(out, conflev=0.95)
+#'
+#'  ##=====================
+#'  ##  Example with starting point based on LTS.
+#'  ##  Extract all subsamples, construct a fan plot specifying the confidence level
+#'  ##  and the initial starting point for monitoring based on p+2 observations,
+#'  ##  strong line width for lines associated with the confidence bands.
+#'      data(wool)
+#'      out <- fsrfan(cycles~., data=wool, init=ncol(wool)+1, nsamp=0, lms=0,
+#'          lwd.env=3, plot=TRUE)
+#'      plot(out, lwd.env=3)
+#'
+#'  ##=====================
+#'  ##  Fan plot using the loyalty cards data.
+#'  ##  In this example, 'la' is the vector contanining the most common values
+#'  ##  of the transformation parameter.
+#'  ##  Store the score test statistics for the specified values of lambda
+#'  ##  and automatically produce the fan plot
+#'      data(loyalty)
+#'      head(loyalty)
+#'      dim(loyalty)
+#'
+#'  ##  la is a vector contanining the most common values of the transformation parameter
+#'      out <- fsrfan(amount_spent~., data=loyalty, la=c(0, 1/3, 0.4, 0.5),
+#'            init=ncol(loyalty)+1, plot=TRUE, lwd=3)
+#'      plot(out, lwd=3)
+#'
+#'  ##  The fan plot shows that even if the third root is the best value of the transformation
+#'  ##  parameter at the end of the search, in earlier steps it lies very close to the upper
+#'  ##  rejection region. The best value of the transformation parameter seems to be the one
+#'  ##  associated with la=0.4, which is always the confidence bands but at the end of search,
+#'  ##  due to the presence of particular observations it goes below the lower rejection line.
+#'
+#'  ##=====================
+#'  ##  Compare BoxCox with Yeo and Johnson transformation.
+#'  ##  Store values of the score test statistic for the five most common
+#'  ##  values of lambda. Produce also a fan plot and display it on the screen.
+#'  ##  Common part to all examples: load wool dataset.
+#'
+#'      data(wool)
+#'
+#'      ##  Store the score test statistic using Box Cox transformation.
+#'      outBC <- fsrfan(cycles~., data=wool, nsamp=0)
+#'
+#'      ##  Store the score test statistic using Yeo and Johnson transformation.
+#'      outYJ <- fsrfan(cycles~., data=wool, family="YJ", nsamp=0)
+#'
+#'      ## Not yet fully implemented
+#'      ##  fanplot(outBC, main="Box Cox")
+#'      ##  fanplot(outYJ,main="Yeo and Johnson")
+#'
+#'      plot(outBC, main="Box Cox")
+#'      plot(outYJ, main="Yeo and Johnson")
+#'
+#'      cat("\nMaximum difference in absolute value: ",
+#'          max(max(abs(outYJ$Score - outBC$Score), na.rm=TRUE)), "\n")
+#'
+#'
+#'
+#'
+#'  ##======================
+#'    ## Call 'fsrfan' with Yeo-Johnson (YJ) transformation
+#'    out <- fsrfan(cycles~., data=wool, family="YJ")
+#'    plot(out)
+#'
+#' }
+#'
+#' @export
+#' @author FSDA team, \email{valentin.todorov@@chello.at}
+
+
+fsrfan <- function(x, ...) UseMethod("fsrfan")
+
+#' @rdname fsrfan
+#' @method fsrfan formula
+#' @param formula a \code{\link{formula}} of the form \code{y ~ x1 + x2 + ...}.
+#' @param data data frame from which variables specified in
+#'    \code{formula} are to be taken.
+#' @param subset an optional vector specifying a subset of observations
+#'    to be used in the fitting process.
+#' @param weights an optional vector of weights to be used
+#"    in the fitting process.
+#'    %%% If specified, weighted least squares is used
+#'    %%% with weights \code{weights} (that is, minimizing \code{sum(w*e^2)});
+#'    %%% otherwise ordinary least squares is used.
+#'   \bold{NOT USED YET}.
+#'
+#' @param na.action a function which indicates what should happen
+#'    when the data contain \code{NA}s.  The default is set by
+#'    the \code{na.action} setting of \code{\link{options}}, and is
+#'    \code{\link{na.fail}} if that is unset.  The \dQuote{factory-fresh}
+#'    default is \code{\link{na.omit}}.  Another possible value is
+#'    \code{NULL}, no action.  Value \code{\link{na.exclude}} can be useful.
+#' @param model \code{\link{logical}} indicating if the
+#'  model frame, is to be returned.
+#' @param x.ret \code{\link{logical}} indicating if the
+#'  the model matrixis to be returned.
+#' @param y.ret \code{\link{logical}} indicating if the
+#'  response is to be returned.
+#' @param contrasts an optional list.  See the \code{contrasts.arg}
+#'  of \code{\link{model.matrix.default}}.
+#' @param offset this can be used to specify an \emph{a priori}
+#'    known component to be included in the linear predictor
+#'    during fitting.  An \code{\link{offset}} term can be included in the
+#"    formula instead or as well, and if both are specified their sum is used.
+
+fsrfan.formula <- function(formula, data, subset, weights, na.action,
+			         model = TRUE, x.ret = FALSE, y.ret = FALSE,
+                     contrasts = NULL, offset, ...)
+{
+    cl <- match.call()
+
+    ## keep only the arguments which should go into the model frame
+    mf <- match.call(expand.dots = FALSE)
+    m <- match(c("formula", "data", "subset", "weights", "na.action",
+                 "offset"), names(mf), 0)
+    mf <- mf[c(1, m)]
+    mf$drop.unused.levels <- TRUE
+    mf[[1]] <- as.name("model.frame")
+    mf <- eval.parent(mf)
+    ##	  if (method == "model.frame") return(mf)
+
+    mt <- attr(mf, "terms")
+    y <- model.response(mf, "numeric") ## was model.extract(mf, "response")
+
+    if (is.empty.model(mt)) { # "y ~ 0" : no coefficients
+	x <- offset <- NULL
+	fit <- list(coefficients = numeric(0), residuals = y,
+		    fitted.values = 0 * y, intercept = TRUE, df.residual = length(y))
+
+	## alpha = alpha from "..."
+	class(fit) <- "fsrfan"
+    }
+    else {
+        w <- model.weights(mf)
+        offset <- model.offset(mf)
+
+	x <- model.matrix(mt, mf, contrasts)
+
+	## Check if there is an intercept in the model.
+	## A formula without intercept looks like this: Y ~ . -1
+	## If so, remove the corresponding column and use intercept=FALSE;
+    ## by default, intercept=TRUE.
+	xint <- match("(Intercept)", colnames(x), nomatch = 0)
+	if(xint)
+	    x <- x[, -xint, drop = FALSE]
+	fit <- fsrfan.default(x, y, intercept=(xint > 0), ...)
+    }
+
+    if(is.null(fit))
+        return(NULL)
+
+    ## 3) return the na.action info
+    fit$na.action <- attr(mf, "na.action")
+    fit$offset <- offset
+
+    ## 4) return the contrasts used in fitting: possibly as saved earlier.
+    fit$contrasts <- attr(x, "contrasts")
+
+    fit$xlevels <- .getXlevels(mt, mf)
+    fit$call <- cl
+    fit$terms <- mt
+    attr(fit$terms, "intercept") <- ifelse(fit$intercept, 1, 0)
+
+    if(model) fit$model <- mf
+    if(x.ret) fit$x <- x # or? if(xint == 0) x else  x[, c(2:p,1), drop=FALSE]
+    if(y.ret) fit$y <- y
+
+    fit
+}
+
+#' @rdname fsrfan
+#' @method fsrfan default
+#'
 #' @param y Response variable. A vector with \code{n} elements that
 #'  contains the response variable.
 #'
@@ -78,10 +359,10 @@
 #'  be set equal to: \code{p+1}, if the sample size is smaller than 40 or
 #'  \code{min(3 * p + 1, floor(0.5 * (n+p+1)))}, otherwise.
 #'
-#' @param plot If \code{plot=FALSE} (default) or \code{plot=0}  no plot is produced.
+#' @param plot If \code{plot=FALSE} (default) no plot is produced.
 #'  If \code{plot=TRUE} a fan plot is shown.
 #'
-#' @param msg  Controls whether to display or not messages on the screen If \code{msg==TRUE} (default)
+#' @param msg  Controls whether to display or not messages on the screen. If \code{msg==TRUE}
 #'  messages are displayed on the screen. If \code{msg=2}, detailed messages are displayed,
 #'  for example the information at iteration level.
 #'
@@ -105,92 +386,19 @@
 #' @param main A label for the title, default is 'Fan plot'
 #' @param xlim Minimum and maximum for the X-axis
 #' @param ylim Minimum and maximum for the Y-axis
-#' @param cex.lab The magnification to be used for x and y labels relative to the current setting of cex
-#' @param cex.axis The magnification to be used for axis annotation relative to the current setting of cex
 #' @param lwd The line width of the curves which contain the score test, a positive number, default is \code{lwd=2}
 #' @param lwd.env The line width of the lines associated with the envelopes, a positive number, default is \code{lwd.env=1}
 #'
 #' @param trace Whether to print intermediate results. Default is \code{trace=FALSE}.
+#' @param \dots potential further arguments passed to lower level functions.
 #'
-#' @return  An S3 object of class \code{\link{fsrfan.object}} will be returned which is basically a list
-#'  containing the following elements:
-#'  \enumerate{
-#'  \item \code{la} vector containing the values of lambda for which fan plot is constructed
-#'  \item \code{bs} matrix of size \code{p X length(la)} containing the units forming
-#'      the initial subset for each value of lambda
-#'  \item \code{Score} a matrix containing the values of the score test for
-#'      each value of the transformation parameter:
-#'      \itemize{
-#'      \item 1st col = fwd search index;
-#'      \item 2nd col = value of the score test in each step of the fwd search for la[1]
-#'      \item ...
-#'      }
-#'  \item \code{Scorep} matrix containing the values of the score test for positive
-#'      observations for each value of the transformation parameter.
-#'
-#'      Note: this output is present only if input option \code{family='YJpn'} or \code{family='YJall'}.
-#'
-#'  \item \code{Scoren} matrix containing the values of the score test for negative observations
-#'      for each value of the transformation parameter.
-#'
-#'      Note: this output is present only if input option 'family' is 'YJpn' or 'YJall'.
-#'
-#'  \item \code{Scoreb} matrix containing the values of the score test for the joint
-#'      presence of both constructed variables (associated with positive and negative
-#'      observations) for each value of the transformation parameter. In this case
-#'      the reference distribution is the \eqn{F} with 2 and \code{subset_size - p}
-#'      degrees of freedom.
-#'
-#'      Note: this output is present only if input option \code{family='YJall'}.
-#'
-#'  \item \code{Un} a three-dimensional array containing \code{length(la)} matrices of
-#'      size \code{retnUn=(n-init) X retpUn=11}. Each matrix contains
-#'      the unit(s) included in the subset at each step in the search associated
-#'      with the corresponding element of \code{la}.
-#'
-#'      REMARK: at each step the new subset is compared with the old subset.
-#'      \code{Un} contains the unit(s) present in the new subset but not in the old one.
-#'  }
-#'
-#' @references
-#'  Atkinson, A.C. and Riani, M. (2000), \emph{Robust Diagnostic Regression Analysis} Springer Verlag, New York.
-#'
-#'  Atkinson, A.C. and Riani, M. (2002), Tests in the fan plot for robust, diagnostic transformations in regression,
-#'      \emph{Chemometrics and Intelligent Laboratory Systems}, \bold{60}, pp. 87--100.
-#'
-#'  Atkinson, A.C. Riani, M. and  Corbellini A. (2019), The analysis of transformations for profit-and-loss data,
-#'      \emph{Journal of the Royal Statistical Society, Series C, "Applied Statistics"}, \bold{69}, pp. 251--275.
-#'      \doi{10.1111/rssc.12389}
-#'
-#'  Atkinson, A.C. Riani, M. and Corbellini A. (2021), The Box-Cox Transformation: Review and Extensions,
-#'      \emph{Statistical Science}, \bold{36}(2), pp. 239--255. \doi{10.1214/20-STS778}.
-#'
-#' @examples
-#'
-#' \dontrun{
-#'    data(wool)
-#'    XX <- wool
-#'    y <- XX[, ncol(XX)]
-#'    X <- XX[, 1:(ncol(XX)-1), drop=FALSE]
-#'
-#'    out <- fsrfan(y, X)                    # call 'fsrfan' with all default parameters
-#'
-#'    out <- fsrfan(y, X, plot=TRUE)         # call 'fsrfan' and produce the plot
-#'
-#'    ## call 'fsrfan' with Yeo-Johnson (YJ) transformation
-#'    out <- fsrfan(y, X, family="YJ", plot=TRUE)
-#'
-#' }
-#'
-#' @export
-#' @author FSDA team, \email{valentin.todorov@@chello.at}
 
-fsrfan <- function(y, x, intercept=TRUE, plot=FALSE,
+
+fsrfan.default <- function(x, y, intercept=TRUE, plot=FALSE,
         family=c("BoxCox", "YJ", "YJpn", "YJall"), la=c(-1, -0.5, 0, 0.5, 1), lms, alpha=0.75, h, init,
-        msg=TRUE, nocheck=FALSE, nsamp=1000, conflev=0.99,
+        msg=FALSE, nocheck=FALSE, nsamp=1000, conflev=0.99,
         xlab, ylab, main, xlim, ylim,
-        cex.lab, cex.axis, lwd=2, lwd.env=1,
-        trace=FALSE)
+        lwd=2, lwd.env=1, trace=FALSE, ...)
 {
     if(is.data.frame(x))
       x <- data.matrix(x)
@@ -271,16 +479,6 @@ fsrfan <- function(y, x, intercept=TRUE, plot=FALSE,
         control$xlimx <- xlim
     if(!missing(ylim))
         control$ylimy <- ylim
-    if(!missing(cex.lab))
-    {
-        control$FontSize <- 12  ## the default
-        control$FontSize <- cex.lab * control$FontSize
-    }
-    if(!missing(cex.axis))
-    {
-        control$SizeAxesNum <- 12  ## the default
-        control$SizeAxesNum <- cex.axis * control$SizeAxesNum
-    }
 
     control$lwd <- lwd
     control$lwdenv <- lwd.env
@@ -355,10 +553,15 @@ fsrfan <- function(y, x, intercept=TRUE, plot=FALSE,
     ## Un is returned as a cell array (a list of matrices). Convert it to
     ##  a tri-dimensional array
     Un <- unwrapComplexNumericCellArray(as.matrix(.jevalArray(arr$get("Un", as.integer(1)))))
-    aUn <- array(dim=c(dim(Un[[1]])[2], dim(Un[[1]])[1], length(Un)))
-    for(ix in 1:length(Un))
-        aUn[,,ix] <- t(Un[[ix]])
-    dimnames(aUn) <- list(aUn[,1,1], c("Step", 1:10), la)
+
+    ## VT::29.03.2022 - return the Un array as NULL, if init=nrow(X), i.e.
+    ##  only one step is conducted
+    if(!is.null(dim(Un[[1]])[2]) && !is.null(dim(Un[[1]])[1])) {
+        aUn <- array(dim=c(dim(Un[[1]])[2], dim(Un[[1]])[1], length(Un)))
+        for(ix in 1:length(Un))
+            aUn[,,ix] <- t(Un[[ix]])
+        dimnames(aUn) <- list(aUn[,1,1], c("Step", 1:10), la)
+    } else aUn <- NULL
 
     X <- if(as.integer(arr$hasField("X", as.integer(1))) != 1) NULL
                 else as.matrix(.jevalArray(arr$get("X", as.integer(1)), "[[D", simplify = TRUE))
@@ -381,3 +584,60 @@ fsrfan <- function(y, x, intercept=TRUE, plot=FALSE,
     class(ans) <- outclass
     return (ans)
 }
+
+#' @rdname fsrfan
+#' @method plot fsrfan
+#' @title FSR fan plots (robust transformations for regression)
+#'
+#' @param conflev Confidence level for the bands (default is 0.99, that is,
+#'  we plot two horizontal lines corresponding to values -2.58 and 2.58).
+#' @param col a vector specifying the colors for the lines, each
+#'  one corresponding to a \code{la} value. if \code{length(col) < length(la)},
+#'  the colors will be recycled.
+#' @param lty a vector specifying the line types for the lines, each
+#'  one corresponding to a \code{la} value. if \code{length(col) < length(la)},
+#'  the colors will be recycled.
+#' @param xlab A label for the X-axis, default is 'Subset size m'
+#' @param ylab A label for the Y-axis, default is 'Score test statistic'
+#' @param main A label for the title, default is 'Fan plot'
+#' @param xlim Minimum and maximum for the X-axis
+#' @param ylim Minimum and maximum for the Y-axis
+#' @param lwd The line width of the curves which contain the score test, a positive number, default is \code{lwd=2}
+#' @param lwd.env The line width of the lines associated with the envelopes, a positive number, default is \code{lwd.env=1}
+#'
+#' @param \dots potential further arguments passed to lower level functions.
+#'
+plot.fsrfan <- function(x, conflev=0.99, xlim, ylim,
+    xlab="Subset of size m", ylab="Score test statistic", main="Fan plot",
+    col, lty, lwd=2.5, lwd.env=1, ...) {
+
+    if(missing(ylim))
+        ylim <- c(min(x$Score[, -1], na.rm=TRUE), max(x$Score[, -1], na.rm=TRUE))
+    if(missing(xlim))
+        xlim <- c(x$Score[1,1]-1, x$Score[nrow(x$Score), 1]+1)
+
+    if(missing(col)) {
+        r <- c(0, 1, 1, 1, 0, 0)
+        g <- c(0, 0, 0, 1, 1, 1)
+        b <- c(1, 0, 1, 0, 0, 1)
+        col <- rgb(r, g, b, names=c("blue", "red", "magenta", "yellow", "green", "cyan"))
+    }
+    col <- rep(col, ceiling(length(x$la)/length(col)))
+    if(missing(lty))
+        lty <- c(1:3, 5)
+    lty <- rep(lty, ceiling(length(x$la)/length(lty)))
+
+    plot(x$Score[, 1], x$Score[, 2], type="n", xlab=xlab, ylab=ylab,
+        main=main, xlim=xlim, ylim=ylim, yaxt="n", ...)
+    axis(2, pretty(x$Score[,-1], 10))
+    for (i in 2:ncol(x$Score)){
+        lines(x$Score[,1], x$Score[, i], type = 'l', col=col[i-1], lty=lty[i-1], lwd=lwd)
+    }
+    text(rep(x$Score[nrow(x$Score), 1], length(x$la)), x$Score[nrow(x$Score), -1], x$la, adj=c(0,0.5))
+
+    quant <- sqrt(qchisq(conflev, 1))
+    abline(h=quant, col="red", lwd=lwd.env)
+    abline(h=-quant, col="red", lwd=lwd.env)
+
+    invisible(x)
+}
